"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Realm = require("realm");
const shared_1 = require("../../shared");
const discovery_1 = require("../../discovery");
const authservice_1 = require("./authservice");
const decorators_1 = require("../../decorators");
const crypto = require("crypto");
exports.UsernamePasswordConstants = {
    saltWorkIterations: 10000,
    keyLength: 512,
    saltLength: 256,
    digest: "sha256"
};
const PasswordSaltHashSchema = {
    name: 'Password',
    primaryKey: 'userId',
    properties: {
        userId: { type: 'string', optional: false },
        salt: { type: 'string', optional: false },
        hash: { type: 'string', optional: false },
    }
};
let UsernamePasswordAuthService = class UsernamePasswordAuthService extends authservice_1.AuthService {
    start(discovery) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            yield _super("start").call(this, discovery);
            this.passwordRealm = new Realm({
                path: `./data/password.realm`,
                schema: [PasswordSaltHashSchema]
            });
        });
    }
    hashPassword(clearTextPassword, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            return shared_1.Promisify(crypto.pbkdf2.bind(crypto), clearTextPassword, salt, exports.UsernamePasswordConstants.saltWorkIterations, exports.UsernamePasswordConstants.keyLength, exports.UsernamePasswordConstants.digest);
        });
    }
    createSalt(saltWorkFactor) {
        return __awaiter(this, void 0, void 0, function* () {
            return shared_1.Promisify(crypto.randomBytes.bind(crypto), saltWorkFactor);
        });
    }
    comparePassword(candidatePassword, salt, hash) {
        return __awaiter(this, void 0, void 0, function* () {
            const hashedCandidatePassword = yield this.hashPassword(candidatePassword, salt);
            return hashedCandidatePassword.equals(hash);
        });
    }
    attemptToRegister(username, password, isAdmin) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            const foundUser = _super("getUser").call(this, username);
            if (foundUser) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    title: 'The provided credentials are invalid or a user already exists.',
                    code: 611
                });
            }
            const salt = yield this.createSalt(exports.UsernamePasswordConstants.saltWorkIterations);
            const hash = yield this.hashPassword(password, salt);
            this.passwordRealm.write(() => {
                this.passwordRealm.create('Password', {
                    salt: salt.toString("base64"),
                    hash: hash.toString("base64"),
                    userId: username
                });
            });
            const storedAuthUser = _super("storeUser").call(this, username, false);
            return {
                userId: storedAuthUser.userId,
                isAdmin: storedAuthUser.isAdmin,
                isCreated: true
            };
        });
    }
    attemptToLogin(username, password) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            const foundUser = _super("getUser").call(this, username);
            if (!foundUser) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    title: 'The provided credentials are invalid or a user does not exist',
                    code: 611
                });
            }
            const foundPassword = this.passwordRealm.objectForPrimaryKey('Password', username);
            const passwordMatch = yield this.comparePassword(password, new Buffer(foundPassword.salt, "base64"), new Buffer(foundPassword.hash, "base64"));
            if (!passwordMatch) {
                throw new shared_1.JSONError({
                    detail: `Given account: ${username}`,
                    title: 'The provided credentials are invalid or a user does not exist'
                });
            }
            return {
                userId: foundUser.userId,
                isAdmin: foundUser.isAdmin,
                isCreated: false
            };
        });
    }
    authenticateOrCreateUser(body) {
        return __awaiter(this, void 0, void 0, function* () {
            let username = "";
            let password = "";
            let register = false;
            let isAdmin = undefined;
            if (body["user_info"]) {
                username = body["data"];
                password = body["user_info"]["password"];
                register = body["user_info"]["register"];
                isAdmin = body["user_info"]["is_admin"] || false;
            }
            else {
                username = body["username"];
                password = body["password"];
                register = body["register"] || false;
                isAdmin = body["is_admin"] || false;
            }
            if (!username) {
                throw new shared_1.JSONError({
                    title: "Your request did not validate because of missing parameter 'username' .",
                    code: 602,
                    path: 'missing-parameters'
                });
            }
            if (!password) {
                throw new shared_1.JSONError({
                    title: "Your request did not validate because of missing parameter 'password'",
                    code: 602,
                    path: 'missing-parameters'
                });
            }
            if (register) {
                return this.attemptToRegister(username, password);
            }
            else {
                return this.attemptToLogin(username, password);
            }
        });
    }
};
__decorate([
    decorators_1.Start(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [discovery_1.Discovery]),
    __metadata("design:returntype", Promise)
], UsernamePasswordAuthService.prototype, "start", null);
UsernamePasswordAuthService = __decorate([
    decorators_1.BaseRoute('/auth'),
    decorators_1.ServiceName('authservice')
], UsernamePasswordAuthService);
exports.UsernamePasswordAuthService = UsernamePasswordAuthService;
//# sourceMappingURL=usernamepassword.js.map